@@ -36,6 +36,16 @@ __BEGIN_DECLS
 #include <IOKit/iokitmig.h>
 __END_DECLS
 
+#ifndef IOUSBLIBDEBUG
+    #define IOUSBLIBDEBUG		0
+#endif
+
+#if IOUSBLIBDEBUG
+    #define DEBUGPRINT(x,...)	printf(x, ##__VA_ARGS__)
+#else
+    #define DEBUGPRINT(x,...)
+#endif
+
 #define connectCheck() do {	    \
     if (!fConnection)		    \
 	return kIOReturnNoDevice;   \
@@ -71,7 +81,7 @@ IOUSBInterfaceClass::IOUSBInterfaceClass()
   fCFSource(0),
   fIsOpen(false)
 {
-    fUSBInterface.pseudoVTable = (IUnknownVTbl *)  &sUSBInterfaceInterfaceV192;
+    fUSBInterface.pseudoVTable = (IUnknownVTbl *)  &sUSBInterfaceInterfaceV197;
     fUSBInterface.obj = this;
 }
 
@@ -101,11 +111,12 @@ IOUSBInterfaceClass::queryInterface(REFIID iid, void **ppv)
         *ppv = &iunknown;
         addRef();
     }
-    else if (CFEqual(uuid, kIOUSBInterfaceInterfaceID) 
-	    || CFEqual(uuid, kIOUSBInterfaceInterfaceID182) 
-	    || CFEqual(uuid, kIOUSBInterfaceInterfaceID183) 
-	    || CFEqual(uuid, kIOUSBInterfaceInterfaceID190) 
-	    || CFEqual(uuid, kIOUSBInterfaceInterfaceID192)) 
+    else if (CFEqual(uuid, kIOUSBInterfaceInterfaceID)
+             || CFEqual(uuid, kIOUSBInterfaceInterfaceID182)
+             || CFEqual(uuid, kIOUSBInterfaceInterfaceID183)
+             || CFEqual(uuid, kIOUSBInterfaceInterfaceID190)
+             || CFEqual(uuid, kIOUSBInterfaceInterfaceID192)
+             || CFEqual(uuid, kIOUSBInterfaceInterfaceID197)) 
     {
         *ppv = &fUSBInterface;
         addRef();
@@ -357,8 +368,10 @@ IOReturn
 IOUSBInterfaceClass::USBInterfaceClose()
 {
     IOReturn		ret;
+#if 0
     LowLatencyUserBufferInfo *	buffer;
     LowLatencyUserBufferInfo *	nextBuffer;
+#endif
     
     allChecks();
 
@@ -382,7 +395,7 @@ IOUSBInterfaceClass::USBInterfaceClose()
     {
         nextBuffer = fUserBufferInfoListHead;
         buffer = fUserBufferInfoListHead;
-        printf("fUserBufferInfoListHead != NULL: %p, next: %p\n",fUserBufferInfoListHead, buffer->nextBuffer);
+        DEBUGPRINT("fUserBufferInfoListHead != NULL: %p, next: %p\n",fUserBufferInfoListHead, buffer->nextBuffer);
         
         // Traverse the list and release memory
         //
@@ -390,7 +403,7 @@ IOUSBInterfaceClass::USBInterfaceClose()
         {
             nextBuffer = buffer->nextBuffer;
             
-            printf("Releasing %p, %p\n", buffer->bufferAddress, buffer );
+            DEBUGPRINT("Releasing %p, %p\n", buffer->bufferAddress, buffer );
             free ( buffer->bufferAddress );
             free ( buffer );
             
@@ -1147,11 +1160,11 @@ IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync(UInt8 pipeRef, void *buf, UInt
     {
         pb.fDataBufferCookie = dataBufferInfo->cookie;
         pb.fDataBufferOffset = (UInt32) buf - (UInt32)dataBufferInfo->bufferAddress;
-        // printf("LLRead: Found Data buffer: offset = %ld, cookie: %ld\n", pb.fDataBufferOffset, dataBufferInfo->cookie);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync  Found Data buffer: offset = %ld, cookie: %ld\n", pb.fDataBufferOffset, dataBufferInfo->cookie);
     }
     else
     {
-        // printf("LL Read: Ooops, couldn't find buffer %p in our list\n",buf);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync Ooops, couldn't find buffer %p in our list\n",buf);
         return kIOUSBLowLatencyBufferNotPreviouslyAllocated;
     }
     
@@ -1162,11 +1175,11 @@ IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync(UInt8 pipeRef, void *buf, UInt
     {
         pb.fFrameListBufferCookie = frameListData->cookie;
         pb.fFrameListBufferOffset = (UInt32) frameList - (UInt32)frameListData->bufferAddress;
-        // printf("LL Read: Found FrameList buffer: offset = %ld, cookie : %ld\n", pb.fFrameListBufferOffset, frameListData->cookie);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync  Found FrameList buffer: offset = %ld, cookie : %ld\n", pb.fFrameListBufferOffset, frameListData->cookie);
     }
     else
     {
-        // printf("LL Read: Ooops, couldn't find buffer %p in our list\n",frameList);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyReadIsochPipeAsync  Ooops, couldn't find buffer %p in our list\n",frameList);
         return kIOUSBLowLatencyFrameListNotPreviouslyAllocated;
     }
 
@@ -1219,11 +1232,11 @@ IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync(UInt8 pipeRef, void *buf, UIn
     {
         pb.fDataBufferCookie = dataBufferInfo->cookie;
         pb.fDataBufferOffset = (UInt32) buf - (UInt32)dataBufferInfo->bufferAddress;
-       //  printf("LLWrite: Found Data buffer: offset = %ld, cookie: %ld\n", pb.fDataBufferOffset, dataBufferInfo->cookie);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync Found Data buffer: offset = %ld, cookie: %ld\n", pb.fDataBufferOffset, dataBufferInfo->cookie);
     }
     else
     {
-       // printf("LLWrite: Ooops, couldn't find buffer %p in our list\n",buf);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync Ooops, couldn't find buffer %p in our list\n",buf);
         return kIOUSBLowLatencyBufferNotPreviouslyAllocated;
     }
     
@@ -1234,11 +1247,11 @@ IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync(UInt8 pipeRef, void *buf, UIn
     {
         pb.fFrameListBufferCookie = frameListData->cookie;
         pb.fFrameListBufferOffset = (UInt32) frameList - (UInt32)frameListData->bufferAddress;
-        // printf("LLWrite: Found FrameList buffer: offset = %ld, cookie = %ld\n", pb.fFrameListBufferOffset, frameListData->cookie);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync Found FrameList buffer: offset = %ld, cookie = %ld\n", pb.fFrameListBufferOffset, frameListData->cookie);
     }
     else
     {
-        // printf("LLWrite: Ooops, couldn't find buffer %p in our list\n",frameList);
+        DEBUGPRINT("IOUSBInterfaceClass::LowLatencyWriteIsochPipeAsync Ooops, couldn't find buffer %p in our list\n",frameList);
         return kIOUSBLowLatencyFrameListNotPreviouslyAllocated;
     }
 
@@ -1271,13 +1284,14 @@ IOUSBInterfaceClass::LowLatencyCreateBuffer( void ** buffer, IOByteCount bufferS
     
     allChecks();
 
-    // printf("In LowLatencyCreateBuffer, size: %d, type, %d\n", (int)bufferSize, (int)bufferType);
+    DEBUGPRINT("IOUSBLib::LowLatencyCreateBuffer size: %d, type, %d\n", (int)bufferSize, (int)bufferType);
     
     // Allocate our buffer Data and zero it
     //
     bufferInfo = ( LowLatencyUserBufferInfo *) malloc( sizeof(LowLatencyUserBufferInfo) );
     if ( bufferInfo == NULL )
     {
+        DEBUGPRINT("IOUSBLib::LowLatencyCreateBuffer:  Could not allocate a LowLatencyUserBufferInfo of %ld bytes\n",sizeof(LowLatencyUserBufferInfo));
         *buffer = NULL;
         result = kIOReturnNoMemory;
         goto ErrorExit;
@@ -1290,11 +1304,12 @@ IOUSBInterfaceClass::LowLatencyCreateBuffer( void ** buffer, IOByteCount bufferS
     *buffer = malloc( bufferSize );
     if ( *buffer == NULL )
     {
+        DEBUGPRINT("IOUSBLib::LowLatencyCreateBuffer:  Could not allocate a buffer of size %ld, type %ld\n", bufferSize, bufferType);
         result = kIOReturnNoMemory;
         goto ErrorExit;
     }
     
-    // printf("\tBuffer: %p\n", *buffer);
+    DEBUGPRINT("IOUSBLib::LowLatencyCreateBuffer Buffer: %p\n", *buffer);
     
     // Update our buffer Data
     //
@@ -1328,6 +1343,7 @@ IOUSBInterfaceClass::LowLatencyCreateBuffer( void ** buffer, IOByteCount bufferS
         
         // Fall through to return result
         //
+        DEBUGPRINT("IOUSBLib::LowLatencyCreateBuffer:  Kernel call to kUSBInterfaceUserClientLowLatencyPrepareBuffer returned 0x%x\n", result);
     }
     
     
@@ -1346,7 +1362,7 @@ IOUSBInterfaceClass::LowLatencyDestroyBuffer( void * buffer )
     
     allChecks();
 
-    //printf("In LowLatencyDestroyBuffer, buffer %p\n", buffer);
+    DEBUGPRINT("IOUSBLib::LowLatencyDestroyBuffer, buffer %p\n", buffer);
     
     // We need to find the LowLatencyUserBufferInfo structure that contains
     // this buffer and then remove it from the list and free the structure
@@ -1355,6 +1371,7 @@ IOUSBInterfaceClass::LowLatencyDestroyBuffer( void * buffer )
     bufferData = FindBufferAddressInList( buffer );
     if ( bufferData == NULL )
     {
+        DEBUGPRINT("IOUSBLib::LowLatencyDestroyBuffer:  Could not find buffer (%p) in our list\n", buffer);
         result = kIOReturnBadArgument;
         goto ErrorExit;
     }
@@ -1364,6 +1381,7 @@ IOUSBInterfaceClass::LowLatencyDestroyBuffer( void * buffer )
     found = RemoveDataBufferFromList( bufferData );
     if ( !found )
     {
+        DEBUGPRINT("IOUSBLib::LowLatencyDestroyBuffer:  Could not remove buffer (%p) from our list\n", buffer);
         result = kIOReturnBadArgument;
         goto ErrorExit;
     }
@@ -1380,6 +1398,10 @@ IOUSBInterfaceClass::LowLatencyDestroyBuffer( void * buffer )
     free ( bufferData->bufferAddress );
     free ( bufferData );
     
+    if ( result != kIOReturnSuccess )
+    {
+        DEBUGPRINT("IOUSBLib::LowLatencyDestroyBuffer:  Kernel call kUSBInterfaceUserClientLowLatencyReleaseBuffer returned 0x%x\n", result);
+    }
 
 ErrorExit:
     
@@ -1553,6 +1575,121 @@ IOUSBInterfaceClass::RemoveDataBufferFromList( LowLatencyUserBufferInfo * remove
 }
 
 
+IOReturn
+IOUSBInterfaceClass::GetBusMicroFrameNumber(UInt64 *microFrame, AbsoluteTime *atTime)
+{
+    IOUSBGetFrameStruct 	stuff;
+    mach_msg_type_number_t 	outSize = sizeof(stuff);
+    IOReturn 			ret;
+
+    connectCheck();
+
+    ret = io_connect_method_scalarI_structureO(fConnection, kUSBInterfaceUserClientGetMicroFrameNumber, NULL, 0, (char *)&stuff, &outSize);
+    if(kIOReturnSuccess == ret)
+    {
+        *microFrame = stuff.frame;
+        *atTime = stuff.timeStamp;
+    }
+    if (ret == MACH_SEND_INVALID_DEST)
+    {
+        fIsOpen = false;
+        fConnection = MACH_PORT_NULL;
+        ret = kIOReturnNoDevice;
+    }
+    return ret;
+}
+
+
+IOReturn
+IOUSBInterfaceClass::GetFrameListTime(UInt32 *microsecondsInFrame)
+{
+    mach_msg_type_number_t 	outSize = 1;
+    IOReturn 			ret;
+
+    connectCheck();
+
+    ret = io_connect_method_scalarI_scalarO(fConnection, kUSBInterfaceUserClientGetFrameListTime, NULL, 0, (int*)microsecondsInFrame, &outSize);
+    if (ret == MACH_SEND_INVALID_DEST)
+    {
+        fIsOpen = false;
+        fConnection = MACH_PORT_NULL;
+        ret = kIOReturnNoDevice;
+    }
+    return ret;
+}
+
+
+IOReturn
+IOUSBInterfaceClass::GetIOUSBLibVersion(NumVersion *ioUSBLibVersion, NumVersion *usbFamilyVersion)
+{
+    CFURLRef    bundleURL;
+    CFBundleRef myBundle;
+    UInt32  	usbFamilyBundleVersion;
+    UInt32  	usbLibBundleVersion;
+    UInt32 * 	tmp;
+    
+    connectCheck();
+
+    // Make a CFURLRef from the CFString representation of the
+    // bundle's path. See the Core Foundation URL Services chapter
+    // for details.
+    bundleURL = CFURLCreateWithFileSystemPath(
+                                              kCFAllocatorDefault,
+                                              CFSTR("/System/Library/Extensions/IOUSBFamily.kext"),
+                                              kCFURLPOSIXPathStyle,
+                                              true );
+
+    // Make a bundle instance using the URLRef.
+    myBundle = CFBundleCreate( kCFAllocatorDefault, bundleURL );
+
+    // Look for the bundle's version number.
+    usbFamilyBundleVersion = CFBundleGetVersionNumber( myBundle );
+
+    // Any CF objects returned from functions with "create" or
+    // "copy" in their names must be released by us!
+    CFRelease( bundleURL );
+    CFRelease( myBundle );
+
+    // Make a CFURLRef from the CFString representation of the
+    // bundle's path. See the Core Foundation URL Services chapter
+    // for details.
+    bundleURL = CFURLCreateWithFileSystemPath(
+                                              kCFAllocatorDefault,
+                                              CFSTR("/System/Library/Extensions/IOUSBFamily.kext/Contents/PlugIns/IOUSBLib.bundle"),
+                                              kCFURLPOSIXPathStyle,
+                                              true );
+
+    // Make a bundle instance using the URLRef.
+    myBundle = CFBundleCreate( kCFAllocatorDefault, bundleURL );
+
+    // Look for the bundle's version number.
+    usbLibBundleVersion = CFBundleGetVersionNumber( myBundle );
+
+    // Any CF objects returned from functions with "create" or
+    // "copy" in their names must be released by us!
+    CFRelease( bundleURL );
+    CFRelease( myBundle );
+
+    // Cast the NumVersion to a UInt32 so we can just copy the data directly in.
+    //
+    if ( ioUSBLibVersion )
+    {
+        tmp = (UInt32 *) ioUSBLibVersion;
+        *tmp = usbLibBundleVersion;
+    }
+
+    if ( usbFamilyVersion )
+    {
+        tmp = (UInt32 *) usbFamilyVersion;
+        *tmp = usbFamilyBundleVersion;
+    }
+    
+    return kIOReturnSuccess;
+
+}
+
+
+
 IOCFPlugInInterface 
 IOUSBInterfaceClass::sIOCFPlugInInterfaceV1 = {
     0,
@@ -1566,8 +1703,8 @@ IOUSBInterfaceClass::sIOCFPlugInInterfaceV1 = {
 };
 
 
-IOUSBInterfaceStruct192 
-IOUSBInterfaceClass::sUSBInterfaceInterfaceV192 = {
+IOUSBInterfaceStruct197 
+IOUSBInterfaceClass::sUSBInterfaceInterfaceV197 = {
     0,
     &IOUSBIUnknown::genericQueryInterface,
     &IOUSBIUnknown::genericAddRef,
@@ -1624,7 +1761,11 @@ IOUSBInterfaceClass::sUSBInterfaceInterfaceV192 = {
     &IOUSBInterfaceClass::interfaceLowLatencyReadIsochPipeAsync,
     &IOUSBInterfaceClass::interfaceLowLatencyWriteIsochPipeAsync,
     &IOUSBInterfaceClass::interfaceLowLatencyCreateBuffer,
-    &IOUSBInterfaceClass::interfaceLowLatencyDestroyBuffer
+    &IOUSBInterfaceClass::interfaceLowLatencyDestroyBuffer,
+    // ---------- new with 1.9.7
+    &IOUSBInterfaceClass::interfaceGetBusMicroFrameNumber,
+    &IOUSBInterfaceClass::interfaceGetFrameListTime,
+    &IOUSBInterfaceClass::interfaceGetIOUSBLibVersion
 };
 
 
@@ -1874,3 +2015,16 @@ IOReturn
 IOUSBInterfaceClass::interfaceLowLatencyDestroyBuffer(void *self, void * buffer )
     { return getThis(self)->LowLatencyDestroyBuffer( buffer); }
 
+//--------------- added in 1.9.7
+IOReturn
+IOUSBInterfaceClass::interfaceGetBusMicroFrameNumber(void *self, UInt64 *microFrame, AbsoluteTime *atTime)
+{ return getThis(self)->GetBusMicroFrameNumber(microFrame, atTime); }
+
+IOReturn
+IOUSBInterfaceClass::interfaceGetFrameListTime(void *self, UInt32 *microsecondsInFrame)
+{ return getThis(self)->GetFrameListTime(microsecondsInFrame); }
+
+IOReturn
+IOUSBInterfaceClass::interfaceGetIOUSBLibVersion( void *self, NumVersion *ioUSBLibVersion, NumVersion *usbFamilyVersion)
+{ return getThis(self)->GetIOUSBLibVersion(ioUSBLibVersion, usbFamilyVersion); }
+
